.TH TERM L "Radiation Oncology, NCMH"
.UC 4
.SH NAME
term \- "crash-proof" datum entry routines using termcap(3x)
.SH SYNOPSIS
#include <term.h>
.sp
.B cc
[ flags ] files
.B \-lterm \-ltermcap
[ libraries ]
.SH DESCRIPTION
These routines look after datum entry for the basic datum
types and provide some cursor control.
A call to
.I InitTerm
initializes the 
.I term
package by 
getting the terminal type from environment variable TERM
and by reading the appropriate 
.I termcap
entry.
In typical use, the calling program would paint a screen
and place the cursor at the beginning of a datum field
using 
.I page, gotoxy,
and
.I printf
and would then call the appropriate 
.I term
routine to enter or edit that datum.
The field is highlighted using underline mode and the default value, if any,
is displayed.
The user may then enter a new value of the datum under strict field size
enforcement or accept the default.
Acceptance of the displayed value is indicated by a RETURN.
The default value can be retrieved by ^X.
No abort is possible.
When the value is accepted, the field is rewritten in normal mode and with the
new value of the datum, and the cursor is left at the end of the field.

Exceptions to the above are
.I GetBool
for which valid inputs are n, N, y, Y, SPACE, and RETURN, for
no, yes, toggle, and accept respectively,
.I PickOne
which understands ^ for previous value, SPACE for next value, and RETURN
for accept,
and
.I MenuPick
which understands ^ for previous value, SPACE for next value, 
> for next page, and RETURN
for accept.

Magic cookies are dealt with
properly but with the result that the 
field on the screen is wider
than the requested datum field
.I "during entry".
The requested field size is used when the field is rewritten
in normal mode after acceptance
of a datum value.
This is a feature of
.I term
and a bug in the terminals with magic cookies.
.I Curses(3x)
deals with this problem by just not allowing standout mode.
Which would you rather buy?
.SH SEE ALSO
termcap(3x)
.br
c_term(l), if you have it
.SH AUTHOR
.nf
George W. Sherouse
Radiation Oncology
North Carolina Memorial Hospital
University of North Carolina, Chapel Hill
.SH FUNCTIONS
Initialize term.
Returns 0 if successful, -1 if getenv fails, -2 if tgetent cannot
open termcap file, -3 if tgetent cannot find entry for terminal.
If the return is not zero, do not pass GO.
.br
	int InitTerm();

Get a floating point number.  Imagine %(field_width).(places)f
in a printf.
.br
	float GetFloat(default, field_width, places);
.br
	float default;
.br
	int field_width;
.br
	int places;

Get an integer.
.br
	int GetInt(default, field_width);
.br
	int default;
.br
	int field_width;

Get a boolean.
.br
	int GetBool(default);
.br
	int default;

Get a string.  String should be at least field_size + 1 long
to accomodate the appended null.  Null_ok is a boolean which
declares whether or not a string of length zero is acceptable.
.br
	void GetString(string, field_size, null_ok);
.br
	char *string;
.br
	int field_size;
.br
	int null_ok;

Get a hospital number in the format 00-00-00.
.br
	void GetHospNum(hosp_num);
.br
	char *hosp_num

Pick one string from a list of strings by displaying one choice at a time in
the field.
.br
	int PickOne(list, choice_count);
.br
	char **list;
.br
	int choice_count;

A full-screen version of PickOne.  The page is cleared, the prompt is displayed
at the top of the page and the first <some number> choices are displayed.
The user may step through the choices with ^ and SPACE.
If there are more than <some number> choices, the user may also change pages
with >.
RETURN accepts currently highlighted value.
.br
	int MenuPick(prompt, list, choice_count);
.br
	char *prompt;
.br
	char **list;
.br
	int choice_count;

Clear the screen.
.br
	void page();

Move cursor.  Coordinates are 1-indexed from upper left.
.br
	void gotoxy(col, line);
.br
	int col;
.br
	int line;

Clear to end of line.
.br
	void ClearEOL();

Print string centered on line.  Mode == 0 requests normal print, == 1 requests
underline, and == 2 requests standout.
.br
	void center(line, string, mode);
.br
	int line;
.br
	char *string;
.br
	int mode;

Set or clear standout mode.  Not very useful at the applications level.
.br
	void StandOut(on_or_off);
.br
	int on_or_off;

Set or clear underline mode.  Not very useful at the applications level.
.br
	void underline(on_or_off);
.br
	int on_or_off;

Issue some backspaces.  Not very interesting at the applications level.
.br
	void TermRewind(count);
.br
	int count;

Place stdin into raw mode with no echo on odd calls and restore original modes
on even calls.
Return the terminal's erase character.
Not intended for use at the applications level.
.br
	char TermSetUp();
.SH BUGS
Many sorts of stupidity are not checked for.
These include but are not limited to string longer than terminal width in
.I center,
line or col out of bounds in
.I center
or
.I gotoxy,
fractional part longer than field_size in
.I GetFloat,
and 0 or negative field_size anywhere.

Some unreasonable assumptions are made regarding sanity in the termcap.
You will know what they are if you violate them.
I apologize in advance.

.I GetFloat
does not support scientific notation.
It also does not let you back up over the decimal point once it has placed
it.
This is no great hardship since you can always ^X to restart.

Some of the code is a little ugly, having been hacked unmercifully through many
revisions.
It does seem to work, though.
